﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Extensions.DependencyInjection;

namespace TodoList2.Persistence
{
    public static class DbInitializer
    {
        public static void Initialize(IServiceProvider serviceProvider)
        {
            TodoListDbContext context = serviceProvider.GetRequiredService<TodoListDbContext>();

            /*
             Adatbázis létrehozása, amennyiben nem létezik. A változásokat nem kezeli. Ha változtattunk az
             adatbázis modelljén (a code-first osztályainkon), akkor az adatbázist törölni kell, hogy itt
             újra létrejöhessen a frissített szerkezettel.
             Törléshez az Sql Server Object Explorer ablak használható a Visual Studioban.
             Itt SQL Server > localdb > databases, vagy valami hasonló útvonalon találjuk az adatbázisainkat.
             */
            context.Database.EnsureCreated();

            // Listák keresése
            if (context.Lists.Any())
            {
                return; // Az adatbázis már inicializálva van.
            }

            IList<List> defaultLists = new List<List>
            {
                new List
                {
                    Name = "Bevásárlás",
                    Items = new List<Item>()
                    {
                        new Item()
                        {
                            Name = "Alma",
                            Deadline = DateTime.Now.AddDays(1)
                        },
                        new Item()
                        {
                            Name = "Körte",
                            Deadline = DateTime.Now.AddDays(1)
                        },
                        new Item()
                        {
                            Name = "Sör",
                            Deadline = DateTime.Now
                        }
                    }
                },
                new List
                {
                    Name = "Beadandók",
                    Items = new List<Item>()
                    {
                        new Item()
                        {
                            Name = "WAF beadandó",
                            Deadline = DateTime.Now.AddDays(7)
                        },
                        new Item()
                        {
                            Name = "MI beadandó",
                            Deadline = DateTime.Now.AddDays(10)
                        }
                    }
                },
                new List
                {
                    Name = "Fejlesztés",
                    Items = new List<Item>()
                    {
                        new Item()
                        {
                            Name = "Felhasználókezelés",
                            Description = "Bejelentkezés és kijelentkezés megvalósítása süti alapú authentikációval.",
                            Deadline = DateTime.Now.AddDays(4)
                        },
                        new Item()
                        {
                            Name = "Validáció",
                            Description =
                                "A nézetmodell validációs annotációkkal történő ellátása és a szabályok teljesülésének ellenőrzése.",
                            Deadline = DateTime.Now.AddHours(4)
                        },
                        new Item()
                        {
                            Name = "Regisztráció",
                            Description = "Regisztráció implementációja email cím megerősítéssel.",
                            Deadline = DateTime.Now.AddDays(2)
                        },
                        new Item()
                        {
                            Name = "Asztali kliens",
                            Description = "Asztali WPF adminisztrációs kliens fejlesztése.",
                            Deadline = DateTime.Now.AddDays(12)
                        }
                    }
                }
            };

            foreach (List list in defaultLists)
                context.Lists.Add(list);

            context.SaveChanges();
        }
    }
}